import fs from 'node:fs';
import { Buffer } from 'node:buffer';
import process from 'node:process';
import { createUnplugin } from 'unplugin';
import { createFilter } from '@rollup/pluginutils';

function check(code, options, length) {
    // check if fold is valid
    for (const [start, end] of options) {
        if (start > end) {
            return {
                valid: false,
                message: `Invalid input: start ${start} > end ${end}`
            };
        }
    }
    // check if fold is in the code
    for (const [start, end] of options) {
        if (start > length || end > length) {
            return {
                valid: false,
                message: `Invalid input: start ${start} or end ${end} > length ${length}`
            };
        }
    }
    return {
        valid: true,
        message: ''
    };
}

function processFold(code, options, length) {
    if (options.length === 0) {
        return code;
    }
    const { valid, message } = check(code, options, length);
    if (!valid) {
        throw new Error(message);
    }
    options.forEach(([start, end]) => {
        // line start end
        // insert // ...
        code.splice(start - 1, end - start + 1, {
            lineNum: start,
            content: '// ...'
        });
    });
    return code;
}

function processTruncate(code, options, length) {
    if (options.length === 0) {
        return code;
    }
    const { valid, message } = check(code, options, length);
    if (!valid) {
        throw new Error(message);
    }
    // sort by start
    options.sort((a, b) => a[0] - b[0]);
    let newCode = [];
    for (const [start, end] of options) {
        newCode = newCode.concat(code.slice(start - 1, end));
    }
    return newCode;
}

const defaultInclude = [
    '**/*.vue',
    '**/*.jsx',
    '**/*.tsx',
    '**/*.astro',
    '**/*.svelte',
    '**/*.html',
];
const unpluginFactory = (options = {}) => {
    return {
        name: 'vite-plugin-code-sample',
        transformInclude(id) {
            return filter(options)(id);
        },
        transform(code, id) {
            return transform(code, id, options);
        }
    };
};
function filter(options) {
    return createFilter(options.include || defaultInclude, options.exclude);
}
function transform(code, id, options) {
    var _a;
    if (/<code-sample/.test(code)) {
        const removeInProd = (_a = options === null || options === void 0 ? void 0 : options.removeInProd) !== null && _a !== void 0 ? _a : true;
        if (removeInProd && process.env.NODE_ENV === 'production') {
            // remove all code-sample tags in production
            return code.replace(/<code-sample[^>]*>[\s\S]*?<\/code-sample>/g, '');
        }
        const fileContent = fs.readFileSync(id, 'utf-8');
        // check if <code-sample> has fold and truncate attribute
        const foldOptRaw = code.match(/fold="([^"]+)"/);
        const truncateOptRaw = code.match(/truncate="([^"]+)"/);
        // parse fold and truncate
        const foldOptions = foldOptRaw ? JSON.parse(foldOptRaw[1]) : [];
        const truncateOptions = truncateOptRaw ? JSON.parse(truncateOptRaw[1]) : [];
        let metaLines = fileContent.split('\n').map((content, index) => ({
            lineNum: index + 1,
            content
        }));
        const length = metaLines.length;
        // process fold and truncate
        metaLines = processFold(metaLines, foldOptions, length);
        metaLines = processTruncate(metaLines, truncateOptions, length);
        const processedCode = metaLines.map(line => line.content).join('\n');
        const base64Content = Buffer.from(processedCode).toString('base64');
        return code.replace(/<code-sample/g, `<code-sample data-sample-code="${base64Content}"`);
    }
    return code;
}
const unplugin = /* #__PURE__ */ createUnplugin(unpluginFactory);

export { unplugin as default, filter, transform, unplugin, unpluginFactory };
